/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: unit1.cpp                                              */
/*                                                                       */
/*  Revision: V2.4     Date: 08.12.2018    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Generator Frequenzprogramm fuer CALLISTO                    */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 04.01.2007
// Updated by: Chr. Monstein 19.01.2007
// Updated by: A. James      06.08.2008     V1.21
// Updated by: A. James      12.01.2009     V2.0
// Updated by: Chr. Monstein 16.06.2009     V2.1
// Updated by: Chr. Monstein 12.09.2011     V2.3
// Updated by: Chr. Monstein 08.12.2018     V2.4
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "Unit2.h"
#include "msgbx.h"
#include "Unit4.h"
#include "string.h"
#include "stdio.h"
#include <fstream.h>
#include <math.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;

long N=13201; // max rows
double *freq, *ampl, *intfreq, *intampl, *fprog, *freqlist, *amplist;
double maxi, mini;
double range = 2500;
double rangemin = 500;
char Header[120];
int rows; // pixels per sweep
bool frqprgsaved = 0;
int zeiger[13201];
int Nchannels, maxchan;  // sweeplength
float F1, F2, Fx; // start- & stop frequency
float lo;
int Fn; // file number
float Flow=45.0, Fhigh=870.0;  // range Callisto
float Fstep=0.0625; // stepsize synthsizer
int PixPerSec=800; // constant sampling rate
int nonlinChannels;
char defaultpath[240];  //ovs path rom cfg
char frqprgpath[240];  //frq pathfrom cfg

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    freq   = new double[N];
    ampl = new double[N];
    freqlist = new double[N];
    amplist = new double[N];
    intfreq = new double[N];
    intampl = new double[N];
    fprog = new double[N];
    Form1->Top=(Form1->Top)+(Form1->Height);
    ReadConfig();
}
//---------------------------------------------------------------------------
//Read the OVS file
//---------------------------------------------------------------------------
void __fastcall TForm1::ReadOverview(char *filename)
{
  char st[80];
  long row=0;
  int channel;

  maxi=-1.0, mini=99999.9;

  ifstream infile(filename);
  infile.getline(st,80); // read header
  while (infile)
  {
    infile.getline(st,80);
    if (infile)
    {
      sscanf(st,"%lf;%lf",&freq[row],&ampl[row]);
      if (ampl[row]  > maxi) maxi = ampl[row];
      if ((ampl[row]  < mini)&&(row>=1)) mini = ampl[row];
      row++;
    }
  }
  infile.close();
  rows = row-1;
  sprintf(st,"Max: %12.6g",maxi);
  Label13->Caption=st;

}
//---------------------------------------------------------------------------
//Frequenzprogramm in neues Array kopieren,
//Frequenzbereich welcher mit Scrollbars ausgewhlt wurde ausschliessen.
void __fastcall TForm1::CopyOverview()
{
  int i=0;
  long row = 0;
  while (row < N)
    {
       //Scrollbarposition 10000mal grsser da Integer.
       if ((10000*freq[row]< (ScrollBar1->Position)) || (10000*freq[row]> (ScrollBar2->Position)))
       {
         freqlist[i] = freq[row];
         amplist[i]  = ampl[row];
         i++;
       }
       row++;
    }
    maxchan=i-1;
}
//---------------------------------------------------------------------------
//Frequenzprogramm generieren
//---------------------------------------------------------------------------
void __fastcall TForm1::SelectChannels(int count)
{
   F1 = atof(Edit2->Text.c_str());
   F2 = (atof(Edit3->Text.c_str()))-((ScrollBar2->Position-ScrollBar1->Position)/10000);
   Nchannels = atoi(Edit1->Text.c_str());
   if (F1>F2) //F1 & F2 Tauschen bei falscher Eingabe
   {
       Fx=F1;
       F1=F2;
       F2=Fx;
   }
   float dF1 = (F2-F1)/ (Nchannels-nonlinChannels); //-10 because of 10channels nonlin start
   float dF2 = (F2-F1)/((Nchannels-nonlinChannels)*8);
   float lo = atof(Edit4->Text.c_str()); // used for up/down-converter
   char st[40];

   sprintf(st,"DeltaF=%6.3f/%6.3f MHz",dF1,dF2);
   Label2->Caption=st;

   int i; // channel counter
   int p; // pixel counter within one channel
   int dP1 = (dF1/Fstep + 0.5);  // typ 32 ... 64
   //int dP2 = (dF2/Fstep + 0.5); //2.5
   int adr =0;
   int adroffset=0;

   if (RadioButton1->Checked) adroffset=(F1-Flow)/Fstep;//IF = RF     Callisto 1:1
   if (RadioButton2->Checked)
   {
     adroffset=((F1-lo)-Flow)/Fstep; //IF = RF-lo  downconverter USB
     //some error checking when RadioB2 Checked
     if((F1-lo)<45.0 || (F2-lo)>870.0)
     {
       ShowMessage("Downconverter if=rf-lo (USB) Error:\nPlease check LO-frequency, rf-startfrequency\nand rf-stopfrequency!\n(rf-LO) must be within the Callisto frequency range\n(45...870MHZ).");
     }

   }
   if (RadioButton3->Checked)
   {
     adroffset=((lo-F1)-Flow)/Fstep; //IF = lo-RF  downconverter LSB
     //some error checking when RadioB3 Checked
     if((lo-F1)>870.0 || (lo-F2)<45.0)
     {
       ShowMessage("Downconverter if=lo-rf (LSB) Error:\nPlease check LO-frequency, rf-startfrequency\nand rf-stopfrequency!\n(LO-rf) must be within the Callisto frequency range\n(45...870MHZ).");
     }
   }
   if (RadioButton4->Checked)
      {
        F1 = F1 + lo;
        F2 = F2 + lo;
        adroffset=(F1-Flow)/Fstep;
        //some error checking when RadioB4 Checked
        //if((lo+F1)<45.0 || (lo+F2)>870.0)
        if((F1)<45.0 || (F2)>870.0)
        {
          ShowMessage("Upconverter if=rf+lo Error:\nPlease check LO-frequency, rf-startfrequency\nand rf-stopfrequency!\n(LO+rf) must be within the Callisto frequency range\n(45...870MHZ).");
        }

      }
   float mini;
   adr=adr+adroffset;

   // non linear start1 of frq program 8channels
   if (RadioButton3->Checked)
   {
      for (i=Nchannels-nonlinChannels; i<Nchannels; i++)
      {  //(z.B. 200-8=192; <200)
         zeiger[i] = adr;
      }
   }
   else
   {
      for (i=0; i<nonlinChannels; i++)
      {
         zeiger[i] = adr;
      }
   }

   // linear rest of frq program
   if (RadioButton3->Checked)
   {
      for (i=0; i<Nchannels-nonlinChannels+1; i++)
      {
         mini = 5000;
         for(p=0; p<dP1; p++)
         {
            if(RadioButton5->Checked) //minimun Detection
            {
              if (adr < 0) adr = 0;
              if (amplist[adr]<mini)
              {
                 mini = amplist[adr];
                 if (adr<maxchan) zeiger[Nchannels-nonlinChannels-i] = adr;
                 else zeiger[Nchannels-nonlinChannels-i] = maxchan-(Nchannels-i);
              }
              adr--;

            }
            if(RadioButton6->Checked) //equally spaced
            {
              if (adr < 0) adr = 0;
              if (p==dP1-1)
              {
                 if (adr<maxchan) zeiger[Nchannels-nonlinChannels-i] = adr;
                 else zeiger[Nchannels-nonlinChannels-i] = maxchan-(Nchannels-nonlinChannels-i-1);
              }
              adr--;
            }
         }
      }
   }


   else
   {
      for (i=nonlinChannels; i<Nchannels; i++)
      {
         mini = 5000;
         for(p=0; p<dP1; p++)
         {
            if(RadioButton5->Checked) //minimun Detection
            {
              if (adr>13200) adr=13200;
              if (amplist[adr]<mini)
              {
                 mini = amplist[adr];
                 if (adr<maxchan) zeiger[i] = adr;
                 else zeiger[i] = maxchan-(Nchannels-i);

              }
              adr++;
            }
            if(RadioButton6->Checked) //equally spaced
            {
              if (p==dP1-1)
              {
                 if (adr<maxchan) zeiger[i] = adr;
                 else zeiger[i] = maxchan-(Nchannels-i);

              }
              adr++;
            }
         }
      }
   }
}
//---------------------------------------------------------------------------
//Frequenzprogramm speichern
//---------------------------------------------------------------------------
void __fastcall TForm1::SaveFrequencyProgram(int count)
{
   char st[240];
   int i, p;
   FILE *outfile;
   float RF, IF;
   Fn = atoi(Edit5->Text.c_str()); //Nummer fr Filename
   sprintf(st,"%sfrq%05i.cfg",frqprgpath,Fn); //Filename
   SaveDialog1->FileName=st;
   if (SaveDialog1->Execute())
   {
     outfile = fopen(SaveDialog1->FileName.c_str(), "w+");
     sprintf(st,"%s",DateToStr(Now()));
     fprintf(outfile,"/*------------------------------------------------------------*/\n");
     fprintf(outfile,"/* (C) Copyright Institute of Astronomy ETHZ 8093 Zurich      */\n");
     fprintf(outfile,"/*------------------------------------------------------------*/\n");
     fprintf(outfile,"/* Programname: FRQ%05i.cfg  [minimum of a spectral overview]*/\n",Fn);
     fprintf(outfile,"/*                                                            */\n");
     fprintf(outfile,"/* Revision: V2.2  Date: %s  Autor: Chr. Monstein     */\n",st);
     fprintf(outfile,"/*                                                            */\n");
     fprintf(outfile,"/* Purpose: Frequencyprogram for Callisto spectrometer        */\n");
     fprintf(outfile,"/* all entries may be edited manually using Notepad...        */\n");
     fprintf(outfile,"/*------------------------------------------------------------*/\n");
     fprintf(outfile,"\n");
     fprintf(outfile,"[target]=CALLISTO\n");
     fprintf(outfile,"\n");
     fprintf(outfile,"[on_line_testpoint_number]=%i\n",Nchannels/2);
     fprintf(outfile,"\n");
     fprintf(outfile,"[number_of_measurements_per_sweep]=%i\n",Nchannels); // nmps
     fprintf(outfile,"[number_of_sweeps_per_second]=%i\n",PixPerSec/Nchannels);        // nsps
     fprintf(outfile,"\n");
     lo = atof(Edit4->Text.c_str()); // used for up/down-converter
     if (RadioButton4->Checked) lo = -1*lo;   // Upconverter: "Negative" lo due to Callisto Software
     fprintf(outfile,"[external_lo]=%f\n",lo);
     fprintf(outfile,"\n");

     for (i=0; i<count; i++)
     {
        if (RadioButton3->Checked)
        {
           p = zeiger[count-1-i];
        }
        else
        {
           p = zeiger[i];
        }
        //if(p>=13200) p=13199;
        IF = freqlist[p]; // frequency from overview (45...870MHz)

        //if (freq[p] ==0.0 && i!=0) {IF = freq[zeiger[i-1]]; freq[p]=freq[zeiger[i-1]];}
        if (RadioButton1->Checked) RF = IF;    // Callisto 1:1
        if (RadioButton2->Checked) RF = IF+lo; // downconverter USB
        if (RadioButton3->Checked) RF = lo-IF; // downconverter LSB
        if (RadioButton4->Checked)
        {
           RF = IF+lo;
        }
        sprintf(st,"[%04i]=%08.3f,0\n",i+1,RF);  // rf
        fprintf(outfile, st);
        fprog[i+1]=IF;

     }
     fclose(outfile);
     frqprgsaved = 1;
     Form4->PaintBox1Click(0);
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FileListBox1DblClick(TObject *Sender)
{
  int count = 0;
  char st[80];
  AnsiString FileExt;

  FileExt = ExtractFileExt(FileListBox1->FileName);
  FileExt = UpperCase(FileExt);
  if ( !strcmp(FileExt.c_str(),".PRN"))
  {
       sprintf(st,"Overview=%i entries",count);
       Label1->Caption=st;
       CopyOverview();
       SelectChannels(count);
       SaveFrequencyProgram(Nchannels);
       if (frqprgsaved == 1) Form3->Show();
  }
  frqprgsaved = 0;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DriveComboBox1Change(TObject *Sender)
{
    DirectoryListBox1->Update();
    FileListBox1->Update();
}
//---------------------------------------------------------------------------
//Check if entries are within allowed values (Sweeplength: 10...400)
//---------------------------------------------------------------------------
void __fastcall TForm1::Edit1Exit(TObject *Sender)
{
  char st[80];
  if (atoi(Edit1->Text.c_str()) > 400)
     {
        sprintf(st,"Sweeplength = %i not allowed.\n Enter value from 10 to 400",(atoi(Edit1->Text.c_str())));
        ShowMessage(st);
        Edit1->Text="400";
        Edit1->SetFocus();
     }
  if (atoi(Edit1->Text.c_str()) < 10)
     {
        sprintf(st,"Sweeplength = %i not allowed.\n Enter value from 10 to 400",(atoi(Edit1->Text.c_str())));
        ShowMessage(st);
        Edit1->Text="10";
        Edit1->SetFocus();
     }
}
//---------------------------------------------------------------------------
//Check if entries are within allowed values (rf-startrequency)
//---------------------------------------------------------------------------
void __fastcall TForm1::Edit2Exit(TObject *Sender)
{
  char st[80];
  if ((atoi(Edit2->Text.c_str()) > 870.0)&&(RadioButton1->Checked))   {
        sprintf(st,"rf-startfrequency %i MHz not allowed.\n Enter value from 45.0 MHz to 870.0 MHz",(atoi(Edit2->Text.c_str())));
        ShowMessage(st);
        Edit2->Text="870.0";
        Edit2->SetFocus();
     }
  if ((atoi(Edit2->Text.c_str()) < 45.0)&&(RadioButton1->Checked))   {
        sprintf(st,"rf-startfrequency %i MHz not allowed.\n Enter value from 45.0 MHz to 870.0 MHz",(atoi(Edit2->Text.c_str())));
        ShowMessage(st);
        Edit2->Text="45.0";
        Edit2->SetFocus();
     }
}
//---------------------------------------------------------------------------
//Check if entries are within allowed values (rf-stopfrequency)
//---------------------------------------------------------------------------
void __fastcall TForm1::Edit3Exit(TObject *Sender)
{
  char st[80];
  if ((atoi(Edit3->Text.c_str()) > 870.0)&&(RadioButton1->Checked))
   {
      sprintf(st,"rf-stopfrequency %i MHz not allowed.\n Enter value from 45.0 MHz to 870.0 MHz",(atoi(Edit3->Text.c_str())));
      ShowMessage(st);
      Edit3->Text="870.0";
      Edit3->SetFocus();
   }
  if ((atoi(Edit3->Text.c_str()) < 45.0)&&(RadioButton1->Checked))
   {
      sprintf(st,"rf-stopfrequency %i MHz not allowed.\n Enter value from 45.0 MHz to 870.0 MHz",(atoi(Edit3->Text.c_str())));
      ShowMessage(st);
      Edit3->Text="45.0";
      Edit3->SetFocus();
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RadioButton1Click(TObject *Sender)
{
  Edit4->Text="0.0";
  Edit4->Font->Color=clGrayText;
  Edit4->ReadOnly=1;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RadioButton2Click(TObject *Sender)
{
  Edit4->ReadOnly=0;
  Edit4->SetFocus();
  Edit4->Font->Color=clWindowText;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RadioButton3Click(TObject *Sender)
{
  Edit4->ReadOnly=0;
  Edit4->SetFocus();
  Edit4->Font->Color=clWindowText;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RadioButton4Click(TObject *Sender)
{
  Edit4->ReadOnly=0;
  Edit4->SetFocus();
  Edit4->Font->Color=clWindowText;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button1Click(TObject *Sender)
{
   Help->Show();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FileListBox1Click(TObject *Sender)
{
  AnsiString FileExt;
  char st[80];
  FileExt = ExtractFileExt(FileListBox1->FileName);
  FileExt = UpperCase(FileExt);
  if ( !strcmp(FileExt.c_str(),".PRN") )
  {
    ReadOverview(FileListBox1->FileName.c_str()); // read file into buffer
    Form4->Show(); // show before plott
    Form4->PaintBox1Click(0);
  }
  if (rows != 13200)
  {
     sprintf(st,"The selected OVS-File is corrupted.\n%i data points are missing!",(13200-rows));
     ShowMessage(st);
  }
  else
  {
     sprintf(st,"Overview=%i entries",rows);
     Label1->Caption=st;
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Edit6Change(TObject *Sender) //Plot Range Maximum
{
    int Size = Edit6->GetTextLen(); // Lnge des Strings in Edit1 ermitteln
    Size++;                      // Platz fr NULL-Zeichen hinzufgen
    char *Buffer = new char[Size];  // Buffer als dynamische Variable definieren
    Edit6->GetTextBuf(Buffer,Size); // Edit6.Text in Buffer ablegen
    range = atof(Buffer);
    delete[] Buffer;
    if ((range>0) && (rows>0) && (Form4->Visible) && (atof(Edit6->Text.c_str()) > atof(Edit7->Text.c_str())))
    {
        Form4->PaintBox1Click(0);
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Edit7Change(TObject *Sender)//Plot Range Minimum
{
    int Size = Edit7->GetTextLen(); // Lnge des Strings in Edit1 ermitteln
    Size++;                      // Platz fr NULL-Zeichen hinzufgen
    char *Buffer = new char[Size];  // Buffer als dynamische Variable definieren
    Edit7->GetTextBuf(Buffer,Size); // Edit6.Text in Buffer ablegen
    rangemin = atof(Buffer);
    delete[] Buffer;
    if ((range>0) && (rows>0) && (Form4->Visible)&& (atof(Edit7->Text.c_str()) < atof(Edit6->Text.c_str())))
    {
        Form4->PaintBox1Click(0);
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button2Click(TObject *Sender)  //INTEGRATE
{
    //Integrate selected Files
    int i;
    int countPrn = 0;
    char st[120];
    char filename[120];
    AnsiString prnfile, FileExt;

    for (i = 0; i < FileListBox1->Items->Count; i++)
    {
        if (FileListBox1->Selected[i])
        {
          prnfile = FileListBox1->Items->Strings[i];
          FileExt = ExtractFileExt(prnfile);
          FileExt = UpperCase(FileExt);
          countPrn++;

          if ( !strcmp(FileExt.c_str(),".PRN") )
          {
           ifstream infile(prnfile.c_str());
           infile.getline(st,120);
           sscanf(st,"%15s",&Header);

           int row = 0;
           while (infile)
           {
              infile.getline(st,80);
              if (infile)
              {
                 sscanf(st,"%lf;%lf",&freq[row],&ampl[row]);
                 intfreq[row] = (intfreq[row] + freq[row]);
                 intampl[row] = (intampl[row] + ampl[row]);
                 row++;
              }
           }
           infile.close();
           Update();
          }
        }
    }

    for (int row = 0; row <= rows; row++)
    {
       intfreq[row] = (intfreq[row]/(double)countPrn);
       intampl[row] = (intampl[row]/(double)countPrn);
    }

    if (countPrn >= 2)
    {
        //WRITE FILE
        FILE *outfile;
        sprintf(filename,"OVS_Integrated(%3u_files).prn",countPrn);
        outfile = fopen(filename, "w+");
        if (outfile)
        {
            sprintf(st,"Frequency[MHz];Amplitude RX1[mV]\n");  // rf
            fprintf(outfile, st); // write header
            for (int row = 0; row <= rows; row++)
            //for (int row = 0; row < rows-1; row++)
            {
               /*if (row==0)
               {
                  sprintf(st,"Frequency[MHz];Amplitude RX1[mV]\n");  // rf
                  fprintf(outfile, st);
               }*/
               sprintf(st,"%8.4lf;%8.4lf\n",intfreq[row],intampl[row]);  // rf
               fprintf(outfile, st);
            }
            fclose(outfile);
            FileListBox1->Update();
        }
        for (int row = 0; row <= rows; row++)
        {
           intfreq[row] = 0;
           intampl[row]  = 0;
        }
    }
    else
    {
        ShowMessage("Please select at least two files from list for Integration!");
        for (int row = 0; row <= rows; row++)
        {
           intfreq[row] = 0;
           intampl[row]  = 0;
        }

    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button3Click(TObject *Sender)  //Delete
{
  AnsiString filenam;
  AnsiString FileExt;
  int lastsel;
  for (int i = 0; i < FileListBox1->Items->Count; i++)
  {
     if (FileListBox1->Selected[i])
     {
        filenam = FileListBox1->Items->Strings[i];//FileListBox1->FileName;
        std::remove(filenam.c_str());
        if (FileListBox1->Items->Count!=0) lastsel = i-1;
     }
  }
  FileListBox1->Update();
  if (lastsel == -1)
  {
     lastsel++;
     FileListBox1->Selected[lastsel] = true;
  }
  else
  {
     FileListBox1->Selected[lastsel] = true;
  }

  FileExt = ExtractFileExt(FileListBox1->FileName);
  FileExt = UpperCase(FileExt);
  FileListBox1Click(0);

  /*if ( !strcmp(FileExt.c_str(),".PRN") )
  {
    ReadFile(FileListBox1->FileName.c_str()); // read file into buffer
    Form4->Show(); // show before plott
    Form4->PaintBox1Click(0);
  } */
  //Label7->Caption=lastsel;//intIcold[200];  //zu testzwecken
}//---------------------------------------------------------------------------
void __fastcall TForm1::ScrollBar1Change(TObject *Sender)
{
    char st[120];
    float pos=ScrollBar1->Position;
    if (ScrollBar1->Position>449999)
    {
      sprintf(st,"%8.4f MHz",(pos/10000));
      Label16->Caption=st;
    }
    else
    {
      sprintf(st,"");
      Label16->Caption=st;
      if(ScrollBar2->Position>449999)ScrollBar1->Position=450000;
    }
    if (ScrollBar2->Position<ScrollBar1->Position) ScrollBar2->Position=ScrollBar1->Position;
    ScrollBar1->Position=ScrollBar1->Position-(ScrollBar1->Position%625);
    Form4->PaintBox1Click(0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ScrollBar2Change(TObject *Sender)
{
    char st[120];
    float pos=ScrollBar2->Position;
    if (ScrollBar2->Position>449999)
    {
      if(ScrollBar1->Position==449999)ScrollBar1->Position=450000;
      sprintf(st,"%8.4f MHz",(pos/10000));
      Label17->Caption=st;
    }
    else
    {
      sprintf(st,"");
      Label17->Caption=st;
    }
    if (ScrollBar1->Position>ScrollBar2->Position) ScrollBar1->Position=ScrollBar2->Position;
    ScrollBar2->Position=ScrollBar2->Position-(ScrollBar2->Position%625);
    Form4->PaintBox1Click(0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
    delete [] freq;     //free(freq);
    delete [] ampl;     //free(ampl);
    delete [] freqlist; //free(freqlist);
    delete [] amplist;  //free(amplist);
    delete [] intfreq;  //free(intfreq);
    delete [] intampl;  //free(intampl);
    delete [] fprog;    //free(fprog);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RedrawButtonClick(TObject *Sender)
{
   Form4->Show(); // show before plott
   Form4->PaintBox1Click(0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::BSavePrgClick(TObject *Sender)
{
   FileListBox1DblClick(0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::BautoscaleClick(TObject *Sender)
{
  Edit6->Text=(RoundTo(maxi,2)+100); //Autoscale max
  Edit7->Text=(RoundTo(mini,2)-100); //Autoscale min
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button4Click(TObject *Sender) //EXIT
{
  Form1->Close();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ReadConfig(void)
{
  char buff[240];
  char tmp[240];
  int filenumber;
  int j=0;
  AnsiString st;

  if (FileExists("params.cfg"))
  {
    ifstream infile("params.cfg");
    while (infile)
    {
        infile.getline(buff,240);
        st = LowerCase(AnsiString(buff));
        if (buff[0]=='[')
        {
          if (strstr(st.c_str(),"[ovspath]=")) sscanf(buff,"[ovspath]=%s",&defaultpath);
          if (strstr(st.c_str(),"[nonlinchans]=")) sscanf(buff,"[nonlinchans]=%i",&nonlinChannels);
          if (strstr(st.c_str(),"[outputpath]=")) sscanf(buff,"[outputpath]=%s",&tmp);
          if (strstr(st.c_str(),"[sweeplength]=")) sscanf(buff,"[sweeplength]=%i",&Nchannels);
          if (strstr(st.c_str(),"[startfreq]=")) sscanf(buff,"[startfreq]=%f",&F1);
          if (strstr(st.c_str(),"[stopfreq]=")) sscanf(buff,"[stopfreq]=%f",&F2);
          if (strstr(st.c_str(),"[localosc]=")) sscanf(buff,"[localosc]=%f",&lo);
          if (strstr(st.c_str(),"[filenumber]=")) sscanf(buff,"[filenumber]=%i",&filenumber);
        }
    }
    infile.close();
    while (j<240)     //replace | with space
    {
       if (defaultpath[j]=='|') defaultpath[j]=' ';
       if (tmp[j]=='|') tmp[j]=' ';
       j++;
    }
    if (Nchannels>400) Nchannels = 400;
    if (Nchannels<10) Nchannels = 10;
    Edit1->Text=Nchannels;
    if (F1>870.0) F1=870.0;
    if (F1<45.0) F1=45.0;
    Edit2->Text=F1;
    if (F2>870.0) F2=870.0;
    if (F2<45.0) F2=45.0;
    Edit3->Text=F2;
    if (lo<0.0) lo=0.0;
    Edit4->Text=lo;
    if (filenumber<0)filenumber=0;
    Edit5->Text=filenumber;
    if (DirectoryExists(defaultpath))
    {
      DirectoryListBox1->Directory=defaultpath;
    }
    else
    {
      ShowMessage("Default OVS Directory defined\nin params.cfg does not exist!");
    }
    if (DirectoryExists(tmp))
    {
      sprintf(frqprgpath,"%s",tmp);
    }
    else
    {
      ShowMessage("Default output Directory defined\nin params.cfg does not exist!");
    }
  }
  else
  {
    ShowMessage("Can't find Configuration\nFile \"params.cfg\"!");
    nonlinChannels=8;
  }
  Edit8->Text=abs(nonlinChannels);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Edit8Change(TObject *Sender)
{
  nonlinChannels = atoi(Edit8->Text.c_str());
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FileListBox1KeyPress(TObject *Sender, char &Key)
{
  int SelectedFile;
  if(Key == 'r' || Key == 'u') //reload
  {
    UpdateList();
  }
  if(Key == 'p') //preview
  {
    PreviewFrqPrg();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FileListBox1KeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
  if(Key == VK_F5) //reload
  {
    UpdateList();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::UpdateList(void)
{
  int SelectedFile;
  SelectedFile=FileListBox1->ItemIndex;
  FileListBox1->Update();
  DirectoryListBox1->Update();
  FileListBox1->Selected[SelectedFile]=true;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::PreviewFrqPrg(void)
{
  int count = 0;
  int i, p;
  float RF, IF;
  char st[80];
  AnsiString FileExt;
  FileExt = ExtractFileExt(FileListBox1->FileName);
  FileExt = UpperCase(FileExt);
  if ( !strcmp(FileExt.c_str(),".PRN"))
  {
       sprintf(st,"Overview=%i entries",count);
       Label1->Caption=st;
       CopyOverview();
       SelectChannels(count);
  }
  for (i=0; i<Nchannels; i++)
  {
    if (RadioButton3->Checked)
    {
       p = zeiger[Nchannels-1-i];
    }
    else
    {
       p = zeiger[i];
    }
    //if(p>=13200) p=13199;
    IF = freqlist[p]; // frequency from overview (45...870MHz)
    //if (freq[p] ==0.0 && i!=0) {IF = freq[zeiger[i-1]]; freq[p]=freq[zeiger[i-1]];}
    fprog[i+1]=IF;
  }
  frqprgsaved = 1;
  Form4->PaintBox1Click(0);
  frqprgsaved = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button5Click(TObject *Sender)
{
  UpdateList();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button6Click(TObject *Sender)
{
  PreviewFrqPrg();
}
//---------------------------------------------------------------------------

